@@ -1,42 +1,52 @@
-#include "options.h"
-#include "util.h"
-#include "libtomcrypt/mycrypt.h"
+/*
+ * Dropbear SSH
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
+/* Buffer handling routines, designed to avoid overflows/using invalid data */
+
+#include "includes.h"
+#include "dbutil.h"
 #include "buffer.h"
 
+/* Prevent integer overflows when incrementing buffer position/length.
+ * Calling functions should check arguments first, but this provides a
+ * backstop */
+#define BUF_MAX_INCR 1000000000
+#define BUF_MAX_SIZE 1000000000
+
+/* avoid excessively large numbers, > ~8192 bits */
+#define BUF_MAX_MPINT (8240 / 8)
+
+/* Create (malloc) a new buffer of size */
 buffer* buf_new(unsigned int size) {
 
-	buffer* ret;
+	buffer* buf;
 	
-	ret = (buffer*)m_malloc(sizeof(buffer));
-	buf_init(ret, size);
-	return ret;
+	if (size > BUF_MAX_SIZE) {
+		dropbear_exit("buf->size too big");
+	}
 
-}
-
-/* free the buffer's data and the buffer itself */
-void buf_free(buffer* buf) {
-
-	buf_clear(buf);
-	m_free(buf);
-}
-
-/* free a buffer's data */
-void buf_clear(buffer* buf) {
-
-	m_free(buf->data);
-
-}
-
-/* overwrite the contents of the buffer to clear it */
-void buf_burn(buffer* buf) {
-	
-	m_burn(buf->data, buf->size);
-
-}
-
-/* initialise an already allocated buffer. The data won't be freed before
- * malloc */
-void buf_init(buffer* buf, unsigned int size) {
+	buf = (buffer*)m_malloc(sizeof(buffer));
 
 	if (size > 0) {
 		buf->data = (unsigned char*)m_malloc(size);
@@ -47,11 +57,33 @@ void buf_init(buffer* buf, unsigned int size) {
 	buf->size = size;
 	buf->pos = 0;
 	buf->len = 0;
+
+	return buf;
+
 }
 
-/* resize a buffer, pos and len will be repositioned if required */
+/* free the buffer's data and the buffer itself */
+void buf_free(buffer* buf) {
+
+	m_free(buf->data)
+	m_free(buf);
+}
+
+/* overwrite the contents of the buffer to clear it */
+void buf_burn(buffer* buf) {
+	
+	m_burn(buf->data, buf->size);
+
+}
+
+/* resize a buffer, pos and len will be repositioned if required when
+ * downsizing */
 void buf_resize(buffer *buf, unsigned int newsize) {
 
+	if (newsize > BUF_MAX_SIZE) {
+		dropbear_exit("buf->size too big");
+	}
+
 	buf->data = m_realloc(buf->data, newsize);
 	buf->size = newsize;
 	buf->len = MIN(newsize, buf->len);
@@ -59,47 +91,46 @@ void buf_resize(buffer *buf, unsigned int newsize) {
 
 }
 
-/* create a copy of buf, allocating required memory etc */
-/* lenonly is a boolean flag specifying whether to set the size of the new
- * buffer to be just the len of the source buffer (1), or the size of the
- * source buffer (0) */
-buffer* buf_newcopy(buffer* buf, int lenonly) {
+/* Create a copy of buf, allocating required memory etc. */
+/* The new buffer is sized the same as the length of the source buffer. */
+buffer* buf_newcopy(buffer* buf) {
 	
 	buffer* ret;
-	unsigned int newsize;
 
-	newsize = lenonly ? buf->len : buf->size;
-	ret = buf_new(newsize);
+	ret = buf_new(buf->len);
 	ret->len = buf->len;
 	memcpy(ret->data, buf->data, buf->len);
 	return ret;
 }
 
+/* Set the length of the buffer */
 void buf_setlen(buffer* buf, unsigned int len) {
 	if (len > buf->size) {
-		dropbear_exit("bad buf_setlen");
+		dropbear_exit("Bad buf_setlen");
 	}
 	buf->len = len;
 }
 
+/* Increment the length of the buffer */
 void buf_incrlen(buffer* buf, unsigned int incr) {
-	if (buf->pos + incr > buf->size) {
-		dropbear_exit("bad buf_incrlen");
+	if (incr > BUF_MAX_INCR || buf->len + incr > buf->size) {
+		dropbear_exit("Bad buf_incrlen");
 	}
 	buf->len += incr;
 }
+/* Set the position of the buffer */
 void buf_setpos(buffer* buf, unsigned int pos) {
 
 	if (pos > buf->len) {
-		dropbear_exit("bad buf_setpos");
+		dropbear_exit("Bad buf_setpos");
 	}
 	buf->pos = pos;
 }
 
 /* increment the postion by incr, increasing the buffer length if required */
 void buf_incrwritepos(buffer* buf, unsigned int incr) {
-	if (buf->pos + incr > buf->size) {
-		dropbear_exit("bad buf_incrwritepos");
+	if (incr > BUF_MAX_INCR || buf->pos + incr > buf->size) {
+		dropbear_exit("Bad buf_incrwritepos");
 	}
 	buf->pos += incr;
 	if (buf->pos > buf->len) {
@@ -110,20 +141,35 @@ void buf_incrwritepos(buffer* buf, unsigned int incr) {
 /* increment the position by incr, negative values are allowed, to
  * decrement the pos*/
 void buf_incrpos(buffer* buf,  int incr) {
-	if (buf->pos + incr > buf->len || buf->pos + incr < 0) {
-		dropbear_exit("bad buf_incrpos");
+	if (incr > BUF_MAX_INCR ||
+			(unsigned int)((int)buf->pos + incr) > buf->len 
+			|| ((int)buf->pos + incr) < 0) {
+		dropbear_exit("Bad buf_incrpos");
 	}
 	buf->pos += incr;
 }
 
+/* Get a byte from the buffer and increment the pos */
 unsigned char buf_getbyte(buffer* buf) {
 
+	/* This check is really just ==, but the >= allows us to check for the
+	 * bad case of pos > len, which should _never_ happen. */
 	if (buf->pos >= buf->len) {
-		dropbear_exit("bad buf_getbyte");
+		dropbear_exit("Bad buf_getbyte");
 	}
 	return buf->data[buf->pos++];
 }
 
+/* Get a bool from the buffer and increment the pos */
+unsigned char buf_getbool(buffer* buf) {
+
+	unsigned char b;
+	b = buf_getbyte(buf);
+	if (b != 0)
+		b = 1;
+	return b;
+}
+
 /* put a byte, incrementing the length if required */
 void buf_putbyte(buffer* buf, unsigned char val) {
 
@@ -134,11 +180,12 @@ void buf_putbyte(buffer* buf, unsigned char val) {
 	buf->pos++;
 }
 
-/* returns an in-place pointer to the buffer, for boundschecking */
+/* returns an in-place pointer to the buffer, checking that
+ * the next len bytes from that position can be used */
 unsigned char* buf_getptr(buffer* buf, unsigned int len) {
 
 	if (buf->pos + len > buf->len) {
-		dropbear_exit("bad buf_getptr");
+		dropbear_exit("Bad buf_getptr");
 	}
 	return &buf->data[buf->pos];
 }
@@ -148,12 +195,12 @@ unsigned char* buf_getptr(buffer* buf, unsigned int len) {
 unsigned char* buf_getwriteptr(buffer* buf, unsigned int len) {
 
 	if (buf->pos + len > buf->size) {
-		dropbear_exit("bad buf_getwriteptr");
+		dropbear_exit("Bad buf_getwriteptr");
 	}
 	return &buf->data[buf->pos];
 }
 
-/* return a null-terminated string, it is malloced, so must be free()ed
+/* Return a null-terminated string, it is malloced, so must be free()ed
  * Note that the string isn't checked for null bytes, hence the retlen
  * may be longer than what is returned by strlen */
 unsigned char* buf_getstring(buffer* buf, unsigned int *retlen) {
@@ -162,7 +209,7 @@ unsigned char* buf_getstring(buffer* buf, unsigned int *retlen) {
 	unsigned char* ret;
 	len = buf_getint(buf);
 	if (len > MAX_STRING_LEN) {
-		dropbear_exit("string too long");
+		dropbear_exit("String too long");
 	}
 
 	if (retlen != NULL) {
@@ -176,6 +223,28 @@ unsigned char* buf_getstring(buffer* buf, unsigned int *retlen) {
 	return ret;
 }
 
+/* Return a string as a newly allocated buffer */
+buffer * buf_getstringbuf(buffer *buf) {
+	buffer *ret;
+	unsigned char* str;
+	unsigned int len;
+	str = buf_getstring(buf, &len);
+	ret = m_malloc(sizeof(*ret));
+	ret->data = str;
+	ret->len = len;
+	ret->size = len;
+	ret->pos = 0;
+	return ret;
+}
+
+/* Just increment the buffer position the same as if we'd used buf_getstring,
+ * but don't bother copying/malloc()ing for it */
+void buf_eatstring(buffer *buf) {
+
+	buf_incrpos( buf, buf_getint(buf) );
+}
+
+/* Get an uint32 from the buffer and increment the pos */
 unsigned int buf_getint(buffer* buf) {
 	unsigned int ret;
 
@@ -196,8 +265,7 @@ void buf_putint(buffer* buf, int unsigned val) {
 void buf_putstring(buffer* buf, const unsigned char* str, unsigned int len) {
 	
 	buf_putint(buf, len);
-	memcpy(buf_getwriteptr(buf, len), str, len);
-	buf_incrwritepos(buf, len);
+	buf_putbytes(buf, str, len);
 
 }
 
@@ -214,11 +282,11 @@ void buf_putbytes(buffer *buf, const unsigned char *bytes, unsigned int len) {
 void buf_putmpint(buffer* buf, mp_int * mp) {
 
 	unsigned int len, pad = 0;
-	TRACE(("enter buf_putmpint"));
+	TRACE2(("enter buf_putmpint"))
 
-	assert(mp != NULL);
+	dropbear_assert(mp != NULL);
 
-	if (SIGN(mp)) {
+	if (SIGN(mp) == MP_NEG) {
 		dropbear_exit("negative bignum");
 	}
 
@@ -226,6 +294,8 @@ void buf_putmpint(buffer* buf, mp_int * mp) {
 	if (USED(mp) == 1 && DIGIT(mp, 0) == 0) {
 		len = 0;
 	} else {
+		/* SSH spec requires padding for mpints with the MSB set, this code
+		 * implements it */
 		len = mp_count_bits(mp);
 		/* if the top bit of MSB is set, we need to pad */
 		pad = (len%8 == 0) ? 1 : 0;
@@ -248,12 +318,12 @@ void buf_putmpint(buffer* buf, mp_int * mp) {
 		buf_incrwritepos(buf, len-pad);
 	}
 
-	TRACE(("leave buf_putmpint"));
+	TRACE2(("leave buf_putmpint"))
 }
 
-/* retrieve an mp_int from the buffer. This will only handle +ve or 0 values, 
- * will fail for -ve since they shouldn't be required here.
- * Returns 0 on success, -1 on failure */
+/* Retrieve an mp_int from the buffer.
+ * Will fail for -ve since they shouldn't be required here.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_getmpint(buffer* buf, mp_int* mp) {
 
 	unsigned int len;
@@ -261,20 +331,22 @@ int buf_getmpint(buffer* buf, mp_int* mp) {
 	
 	if (len == 0) {
 		mp_zero(mp);
-		return 0;
+		return DROPBEAR_SUCCESS;
+	}
+
+	if (len > BUF_MAX_MPINT) {
+		return DROPBEAR_FAILURE;
 	}
 
 	/* check for negative */
 	if (*buf_getptr(buf, 1) & (1 << (CHAR_BIT-1))) {
-		dropbear_msg("negative bignum");
-		return -1;
+		return DROPBEAR_FAILURE;
 	}
 
 	if (mp_read_unsigned_bin(mp, buf_getptr(buf, len), len) != MP_OKAY) {
-		dropbear_msg("mpint error");
-		return -1;
+		return DROPBEAR_FAILURE;
 	}
 
 	buf_incrpos(buf, len);
-	return 0;
+	return DROPBEAR_SUCCESS;
 }
