@@ -7,10 +7,11 @@ int main(void)
    mp_digit mp;
    long x, y;
 
+   srand(time(NULL));
    mp_init_multi(&modulus, &R, &p, &pp, NULL);
 
    /* loop through various sizes */
-   for (x = 4; x < 128; x++) {
+   for (x = 4; x < 256; x++) {
        printf("DIGITS == %3ld...", x); fflush(stdout);
        
        /* make up the odd modulus */
@@ -22,7 +23,7 @@ int main(void)
        mp_montgomery_setup(&modulus, &mp);
        
        /* now run through a bunch tests */
-       for (y = 0; y < 100000; y++) {
+       for (y = 0; y < 1000; y++) {
            mp_rand(&p, x/2);        /* p = random */
            mp_mul(&p, &R, &pp);     /* pp = R * p */
            mp_montgomery_reduce(&pp, &modulus, mp);
@@ -43,3 +44,7 @@ int main(void)
 
 
 
+
+/* $Source: /cvs/libtom/libtommath/etc/mont.c,v $ */
+/* $Revision: 1.2 $ */
+/* $Date: 2005/05/05 14:38:47 $ */
